@@ -1,13 +1,14 @@
 /***************************************************************************
- $RCSfile: bankinfoplugin.h,v $
+ $RCSfile$
  -------------------
- cvs         : $Id: bankinfoplugin.h,v 1.1 2004/09/09 20:27:34 aquamaniac Exp $
+ cvs         : $Id$
  begin       : Mon Mar 01 2004
  copyright   : (C) 2004 by Martin Preuss
  email       : martin@libchipcard.de
 
  ***************************************************************************
- *          Please see toplevel file COPYING for license details           *
+ * This file is part of the project "AqBanking".                           *
+ * Please see toplevel file COPYING of that project for license details.   *
  ***************************************************************************/
 
 
