@@ -54,7 +54,7 @@ AH_JOB *AH_Job_GetKeys_new(AB_USER *u){
   int version;
 
   assert(u);
-  j=AH_Job_new("JobGetKeys", u, 0, 0);
+  j=AH_Job_new("JobGetKeys", u, 0, 0, 0);
   if (!j) {
     DBG_ERROR(AQHBCI_LOGDOMAIN,
               "JobGetKeys not supported, should not happen");
@@ -111,6 +111,7 @@ void GWENHYWFAR_CB GWENHYWFAR_CB AH_Job_GetKeys_FreeData(void *bp, void *p){
   GWEN_Crypt_Token_KeyInfo_free(jd->cryptKeyInfo);
   GWEN_Crypt_Token_KeyInfo_free(jd->authKeyInfo);
   free(jd->peerId);
+  GWEN_FREE_OBJECT(jd);
 }
 
 
@@ -350,9 +351,9 @@ AH_JOB *AH_Job_SendKeys_new(AB_USER *u,
   assert(u);
 
   if (authKeyInfo)
-    j=AH_Job_new("JobSendKeysWithAuthKey", u, 0, 0);
+    j=AH_Job_new("JobSendKeysWithAuthKey", u, 0, 0, 0);
   else
-    j=AH_Job_new("JobSendKeys", u, 0, 0);
+    j=AH_Job_new("JobSendKeys", u, 0, 0, 0);
   if (!j) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "JobSendKeys not supported, should not happen");
     return NULL;
@@ -596,7 +597,7 @@ AH_JOB *AH_Job_UpdateBank_new(AB_USER *u) {
   AH_JOB_UPDATEBANK *jd;
 
   assert(u);
-  j=AH_Job_new("JobUpdateBankInfo", u, 0, 0);
+  j=AH_Job_new("JobUpdateBankInfo", u, 0, 0, 0);
   if (!j) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "JobUpdateBankInfo not supported, should not happen");
     return 0;
@@ -628,6 +629,7 @@ void GWENHYWFAR_CB AH_Job_UpdateBank_FreeData(void *bp, void *p){
 
   jd=(AH_JOB_UPDATEBANK*)p;
   AB_Account_List2_FreeAll(jd->accountList);
+  GWEN_FREE_OBJECT(jd);
 }
 
 
@@ -769,7 +771,7 @@ AH_JOB *AH_Job_GetSysId_new(AB_USER *u){
   AH_JOB_GETSYSID *jd;
 
   assert(u);
-  j=AH_Job_new("JobSync", u, 0, 0);
+  j=AH_Job_new("JobSync", u, 0, 0, 0);
   if (!j) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "JobSync not supported, should not happen");
     return 0;
@@ -928,9 +930,9 @@ AH_JOB *AH_Job_TestVersion_new(AB_USER *u, int anon){
 
   assert(u);
   if (anon)
-    j=AH_Job_new("JobDialogInitAnon", u, 0, 0);
+    j=AH_Job_new("JobDialogInitAnon", u, 0, 0, 0);
   else
-    j=AH_Job_new("JobDialogInit", u, 0, 0);
+    j=AH_Job_new("JobDialogInit", u, 0, 0, 0);
   if (!j) {
     DBG_ERROR(AQHBCI_LOGDOMAIN,
               "JobTestVersion not supported, should not happen");
@@ -1080,7 +1082,7 @@ AH_JOB *AH_Job_GetStatus_new(AB_USER *u,
   AH_JOB_GETSTATUS *aj;
   GWEN_DB_NODE *dbArgs;
 
-  j=AH_Job_new("JobGetStatus", u, 0, 0);
+  j=AH_Job_new("JobGetStatus", u, 0, 0, 0);
   if (!j)
     return 0;
 
@@ -1195,7 +1197,7 @@ AH_JOB *AH_Job_Tan_new(AB_USER *u, int process, int jobVersion) {
   const char *s;
 
   DBG_INFO(AQHBCI_LOGDOMAIN, "Looking for TAN job in version %d", jobVersion);
-  j=AH_Job_new("JobTan", u, 0, jobVersion);
+  j=AH_Job_new("JobTan", u, 0, 0, jobVersion);
   if (!j) {
     DBG_INFO(AQHBCI_LOGDOMAIN, "TAN job in version %d not found", jobVersion);
     return NULL;
@@ -1299,8 +1301,16 @@ int AH_Job_Tan_Process(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx){
 	if (p && l) {
 	  GWEN_BUFFER *bbuf;
 
-	  bbuf=GWEN_Buffer_new(0, 256, 0, 1);
+          DBG_INFO(AQHBCI_LOGDOMAIN, "Job has a challengeHHD string:");
+          GWEN_Text_LogString((const char*)p, l, AQHBCI_LOGDOMAIN, GWEN_LoggerLevel_Info);
+
+          bbuf=GWEN_Buffer_new(0, 256, 0, 1);
+#if 0
           GWEN_Text_ToHexBuffer((const char*) p, l, bbuf, 0, 0, 0);
+#else
+          /* data is a string, no need to transform to hex */
+          GWEN_Buffer_AppendBytes(bbuf, (const char*) p, l);
+#endif
 	  free(aj->challengeHhd);
 	  aj->challengeHhd=strdup(GWEN_Buffer_GetStart(bbuf));
 	  GWEN_Buffer_free(bbuf);
@@ -1671,7 +1681,7 @@ AH_JOB *AH_Job_GetItanModes_new(AB_USER *u){
   AH_JOB_GETITANMODES *jd;
 
   assert(u);
-  j=AH_Job_new("JobGetItanModes", u, 0, 0);
+  j=AH_Job_new("JobGetItanModes", u, 0, 0, 0);
   if (!j) {
     DBG_ERROR(AQHBCI_LOGDOMAIN,
               "JobGetItanModes not supported, should not happen");
@@ -1803,7 +1813,7 @@ AH_JOB *AH_Job_ChangePin_new(AB_USER *u, const char *newPin){
   GWEN_DB_NODE *dbArgs;
 
   assert(u);
-  j=AH_Job_new("JobChangePin", u, 0, 0);
+  j=AH_Job_new("JobChangePin", u, 0, 0, 0);
   if (!j) {
     DBG_ERROR(AQHBCI_LOGDOMAIN, "JobChangePin not supported, should not happen");
     return 0;
@@ -1829,3 +1839,192 @@ AH_JOB *AH_Job_ChangePin_new(AB_USER *u, const char *newPin){
 
 
 
+/* __________________________________________________________________________
+ * AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
+ *                             AH_Job_GetAccountSepaInfo
+ * YYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYY
+ */
+
+
+
+GWEN_INHERIT(AH_JOB, AH_JOB_GETACCSEPAINFO)
+
+/* --------------------------------------------------------------- FUNCTION */
+AH_JOB *AH_Job_GetAccountSepaInfo_new(AB_USER *u, AB_ACCOUNT *acc) {
+  AH_JOB *j;
+  GWEN_DB_NODE *dbArgs;
+  AH_JOB_GETACCSEPAINFO *jd;
+  const char *s;
+
+  assert(u);
+  j=AH_Job_new("JobGetAccountSepaInfo", u, 0, 0, 0);
+  if (!j) {
+    DBG_ERROR(AQHBCI_LOGDOMAIN, "JobGetAccountSepaInfo not supported, should not happen");
+    return 0;
+  }
+
+  GWEN_NEW_OBJECT(AH_JOB_GETACCSEPAINFO, jd);
+  GWEN_INHERIT_SETDATA(AH_JOB, AH_JOB_GETACCSEPAINFO, j, jd,
+                       AH_Job_GetAccountSepaInfo_FreeData)
+  AH_Job_SetProcessFn(j, AH_Job_GetAccountSepaInfo_Process);
+
+  jd->account=acc;
+
+  /* set arguments */
+  dbArgs=AH_Job_GetArguments(j);
+  assert(dbArgs);
+
+  s=AB_Account_GetAccountNumber(jd->account);
+  if (s && *s)
+    GWEN_DB_SetCharValue(dbArgs, GWEN_DB_FLAGS_DEFAULT, "accountId", s);
+
+  s=AB_Account_GetSubAccountId(jd->account);
+  if (s && *s)
+    GWEN_DB_SetCharValue(dbArgs, GWEN_DB_FLAGS_DEFAULT, "accountSubId", s);
+
+  s=AB_Account_GetBankCode(jd->account);
+  if (s && *s)
+    GWEN_DB_SetCharValue(dbArgs, GWEN_DB_FLAGS_DEFAULT, "bankCode", s);
+
+  GWEN_DB_SetIntValue(dbArgs, GWEN_DB_FLAGS_DEFAULT,
+                      "country", 280);
+
+  s=AB_Account_GetIBAN(jd->account);
+  if (s && *s)
+    GWEN_DB_SetCharValue(dbArgs, GWEN_DB_FLAGS_DEFAULT, "iban", s);
+
+  s=AB_Account_GetBIC(jd->account);
+  if (s && *s)
+    GWEN_DB_SetCharValue(dbArgs, GWEN_DB_FLAGS_DEFAULT, "bic", s);
+
+
+  DBG_INFO(AQHBCI_LOGDOMAIN, "JobGetAccountSepaInfo created");
+  return j;
+}
+
+
+
+void GWENHYWFAR_CB AH_Job_GetAccountSepaInfo_FreeData(void *bp, void *p){
+  AH_JOB_GETACCSEPAINFO *jd;
+
+  jd=(AH_JOB_GETACCSEPAINFO*)p;
+  GWEN_FREE_OBJECT(jd);
+}
+
+
+
+int AH_Job_GetAccountSepaInfo_Process(AH_JOB *j, AB_IMEXPORTER_CONTEXT *ctx){
+  AH_JOB_GETACCSEPAINFO *jd;
+  GWEN_DB_NODE *dbResponses;
+  GWEN_DB_NODE *dbCurr;
+  AB_USER *u;
+  AB_BANKING *ab;
+
+  assert(j);
+  jd=GWEN_INHERIT_GETDATA(AH_JOB, AH_JOB_GETACCSEPAINFO, j);
+  assert(jd);
+
+  if (jd->scanned)
+    return 0;
+
+  jd->scanned=1;
+
+  dbResponses=AH_Job_GetResponses(j);
+  assert(dbResponses);
+
+  u=AH_Job_GetUser(j);
+  assert(u);
+
+  ab=AH_Job_GetBankingApi(j);
+  assert(ab);
+
+  /* search for "GetAccountSepaInfoResponse" */
+  dbCurr=GWEN_DB_GetFirstGroup(dbResponses);
+  while(dbCurr) {
+    GWEN_DB_NODE *dbXA;
+    int rv;
+
+    rv=AH_Job_CheckEncryption(j, dbCurr);
+    if (rv) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Compromised security (encryption)");
+      AH_Job_SetStatus(j, AH_JobStatusError);
+      return rv;
+    }
+    rv=AH_Job_CheckSignature(j, dbCurr);
+    if (rv) {
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Compromised security (signature)");
+      AH_Job_SetStatus(j, AH_JobStatusError);
+      return rv;
+    }
+
+    dbXA=GWEN_DB_GetGroup(dbCurr, GWEN_PATH_FLAGS_NAMEMUSTEXIST,
+                          "data/GetAccountSepaInfoResponse");
+    if (dbXA) {
+      const char *accountId;
+      const char *bankCode;
+      const char *accountSuffix;
+      const char *iban;
+      const char *bic;
+      const char *sSepa;
+      int useWithSepa=0;
+
+      /* account data found */
+      DBG_INFO(AQHBCI_LOGDOMAIN, "Found a GetAccountSepaInfoResponse segment");
+      accountId=GWEN_DB_GetCharValue(dbXA, "accountId", 0, 0);
+      accountSuffix=GWEN_DB_GetCharValue(dbXA, "accountsubid", 0, 0);
+      bankCode=GWEN_DB_GetCharValue(dbXA, "bankCode", 0, 0);
+      sSepa=GWEN_DB_GetCharValue(dbXA, "sepa", 0, "n");
+      if (strcasecmp(sSepa, "j")==0)
+        useWithSepa=1;
+
+      iban=GWEN_DB_GetCharValue(dbXA, "iban", 0, 0);
+      bic=GWEN_DB_GetCharValue(dbXA, "bic", 0, 0);
+
+      rv=AB_Banking_BeginExclUseAccount(ab, jd->account);
+      if (rv<0) {
+        DBG_ERROR(AQHBCI_LOGDOMAIN, "Unable to lock account");
+      }
+      else {
+        if (accountSuffix)
+          AB_Account_SetSubAccountId(jd->account, accountSuffix);
+        AH_Account_AddFlags(jd->account, AH_BANK_FLAGS_KTV2); /* we have a sub id (even if emtpy), set flag */
+
+	if (useWithSepa) {
+	  DBG_INFO(AQHBCI_LOGDOMAIN, "SEPA available with this account");
+	  AH_Account_AddFlags(jd->account, AH_BANK_FLAGS_SEPA); /* we have a sub id (even if emtpy), set flag */
+	}
+	else {
+	  DBG_INFO(AQHBCI_LOGDOMAIN, "SEPA not available with this account");
+	  AH_Account_SubFlags(jd->account, AH_BANK_FLAGS_SEPA); /* we have a sub id (even if emtpy), set flag */
+	}
+
+        if (iban && *iban) {
+	  if (bic && *bic) {
+	    DBG_NOTICE(AQHBCI_LOGDOMAIN, "Setting IBAN and BIC: %s/%s", iban, bic);
+	    AB_Account_SetIBAN(jd->account, iban);
+	    AB_Account_SetBIC(jd->account, bic);
+	  }
+	  else {
+	    DBG_NOTICE(AQHBCI_LOGDOMAIN, "Setting IBAN (no BIC): %s", iban);
+	    AB_Account_SetIBAN(jd->account, iban);
+	  }
+        }
+        else {
+          DBG_ERROR(AQHBCI_LOGDOMAIN, "Missing information in account: BLZ=[%s], Kto=[%s], IBAN=[%s], BIC=[%s]",
+                    bankCode?bankCode:"",
+                    accountId?accountId:"",
+                    iban?iban:"",
+                    bic?bic:"");
+        }
+        AB_Banking_EndExclUseAccount(ab, jd->account, 0);
+      }
+    }
+
+    dbCurr=GWEN_DB_GetNextGroup(dbCurr);
+  }
+
+  return 0;
+}
+
+
+
