@@ -16,6 +16,9 @@
 
 
 #include "globals.h"
+/* The code below uses the private symbol AH_Provider_SendDtazv from libaqhbci */
+#include "hbci_l.h"
+#include "provider_l.h"
 
 #include <gwenhywfar/text.h>
 #include <gwenhywfar/fslock.h>
@@ -34,20 +37,6 @@
 #endif
 
 
-/* this is a more or less secret job which is not declared in AqHBCI, so
- * we must do it here. The AQHBCI_API makes this file import that symbol
- * from the libaqhbci library.
- */
-AQHBCI_API
-int AH_Provider_SendDtazv(AB_PROVIDER *pro,
-			  AB_ACCOUNT *a,
-			  AB_IMEXPORTER_CONTEXT *ctx,
-			  const uint8_t *dataPtr,
-			  uint32_t dataLen,
-			  int nounmount,
-			  uint32_t guiid);
-
-
 static int _incrementUniqueId(AB_ACCOUNT *a, const char *path) {
   GWEN_BUFFER *pbuf;
   const char *s;
@@ -321,6 +310,7 @@ int sendDtazv(AB_BANKING *ab,
   dtazv=GWEN_Buffer_new(0, 1024, 0, 1);
   /* Read DTAZV file */
   if (_readFile(inFile, dtazv)) {
+    GWEN_Buffer_free(dtazv);
     return 3;
   }
 
@@ -370,7 +360,7 @@ int sendDtazv(AB_BANKING *ab,
     rv=AH_Provider_SendDtazv(pro, a, ctx,
 			     (const uint8_t*)GWEN_Buffer_GetStart(dtazv),
 			     GWEN_Buffer_GetUsedBytes(dtazv),
-			     0, 0);
+			     1, 1, 0);
     /* write ctx file */
     if (ctxFile) {
       GWEN_DB_NODE *dbCtx;
@@ -388,6 +378,7 @@ int sendDtazv(AB_BANKING *ab,
 		    ctxFile, rv2);
 	}
       }
+      GWEN_DB_Group_free(dbCtx);
     }
 
     AB_ImExporterContext_free(ctx);
