@@ -39,13 +39,11 @@ int transfers(AB_BANKING *ab,
   const char *profileFile;
   const char *bankId;
   const char *accountId;
-  const char *country;
   int forceCheck;
   int fillGaps;
   AB_IMEXPORTER_CONTEXT *ctx=0;
   AB_IMEXPORTER_ACCOUNTINFO *iea;
   AB_JOB_LIST2 *jobList;
-  AB_TRANSACTION_LIST2 *tlist=NULL;
   int rvExec;
   const GWEN_ARGS args[]={
   {
@@ -191,7 +189,6 @@ int transfers(AB_BANKING *ab,
   ctxFile=GWEN_DB_GetCharValue(db, "ctxfile", 0, 0);
   inFile=GWEN_DB_GetCharValue(db, "inFile", 0, 0);
   fillGaps=GWEN_DB_GetIntValue(db, "fillGaps", 0, 0);
-  country=GWEN_DB_GetCharValue(db, "country", 0, "de");
 
   rv=AB_Banking_Init(ab);
   if (rv) {
@@ -245,7 +242,7 @@ int transfers(AB_BANKING *ab,
       DBG_ERROR(0, "Account %s/%s not found, aborting",
 		AB_ImExporterAccountInfo_GetBankCode(iea),
 		AB_ImExporterAccountInfo_GetAccountNumber(iea));
-      AB_Transaction_List2_free(tlist);
+      AB_Job_List2_FreeAll(jobList);
       AB_ImExporterContext_free(ctx);
       return 3;
     }
@@ -254,19 +251,15 @@ int transfers(AB_BANKING *ab,
 
       t=AB_ImExporterAccountInfo_GetFirstTransaction(iea);
       while(t) {
-	const char *rCountry;
 	const char *rBankId;
 	const char *rAccountId;
 	AB_BANKINFO_CHECKRESULT res;
 	AB_JOB *j;
 
-	rCountry=AB_Transaction_GetRemoteCountry(t);
-	if (rCountry==NULL)
-          rCountry=country;
         rBankId=AB_Transaction_GetRemoteBankCode(t);
 	rAccountId=AB_Transaction_GetRemoteAccountNumber(t);
 	res=AB_Banking_CheckAccount(ab,
-				    rCountry,
+				    "de",
 				    0,
 				    rBankId,
 				    rAccountId);
@@ -276,7 +269,7 @@ int transfers(AB_BANKING *ab,
 		    "Invalid combination of bank code and account number "
 		    "for remote account (%s/%s)",
 		    rBankId, rAccountId);
-	  AB_Transaction_List2_free(tlist);
+	  AB_Job_List2_FreeAll(jobList);
 	  AB_ImExporterContext_free(ctx);
 	  return 3;
 
@@ -284,7 +277,7 @@ int transfers(AB_BANKING *ab,
 	  DBG_ERROR(0, "Remote bank code is unknown (%s/%s)",
 		    rBankId, rAccountId);
 	  if (forceCheck) {
-	    AB_Transaction_List2_free(tlist);
+	    AB_Job_List2_FreeAll(jobList);
 	    AB_ImExporterContext_free(ctx);
 	    return 4;
 	  }
@@ -301,7 +294,7 @@ int transfers(AB_BANKING *ab,
 
 	default:
 	  DBG_ERROR(0, "Unknown check result %d", res);
-	  AB_Transaction_List2_free(tlist);
+	  AB_Job_List2_FreeAll(jobList);
 	  AB_ImExporterContext_free(ctx);
 	  return 4;
 	}
@@ -314,19 +307,21 @@ int transfers(AB_BANKING *ab,
 	rv=AB_Job_CheckAvailability(j);
 	if (rv<0) {
 	  DBG_ERROR(0, "Job not supported.");
-	  AB_Transaction_List2_free(tlist);
-	  AB_ImExporterContext_free(ctx);
-	  return 3;
-	}
-	rv=AB_JobSingleTransfer_SetTransaction(j, t);
-	if (rv<0) {
-	  DBG_ERROR(0, "Unable to add transaction for account %s/%s, aborting",
-		    AB_ImExporterAccountInfo_GetBankCode(iea),
-		    AB_ImExporterAccountInfo_GetAccountNumber(iea));
-	  AB_Transaction_List2_free(tlist);
+	  AB_Job_free(j);
+	  AB_Job_List2_FreeAll(jobList);
 	  AB_ImExporterContext_free(ctx);
 	  return 3;
 	}
+	rv=AB_Job_SetTransaction(j, t);
+        if (rv<0) {
+          DBG_ERROR(0, "Unable to add transaction for account %s/%s, aborting",
+                    AB_ImExporterAccountInfo_GetBankCode(iea),
+                    AB_ImExporterAccountInfo_GetAccountNumber(iea));
+          AB_Job_free(j);
+          AB_Job_List2_FreeAll(jobList);
+          AB_ImExporterContext_free(ctx);
+          return 3;
+        }
 	AB_Job_List2_PushBack(jobList, j);
 	t=AB_ImExporterAccountInfo_GetNextTransaction(iea);
       } /* while t */
@@ -336,8 +331,6 @@ int transfers(AB_BANKING *ab,
   } /* while */
   AB_ImExporterContext_free(ctx);
 
-  AB_Transaction_List2_free(tlist);
-
   /* execute jobs */
   rvExec=0;
   ctx=AB_ImExporterContext_new();
@@ -346,6 +339,7 @@ int transfers(AB_BANKING *ab,
     fprintf(stderr, "Error on executeQueue (%d)\n", rv);
     rvExec=3;
   }
+  AB_Job_List2_FreeAll(jobList);
 
   /* write context */
   rv=writeContext(ctxFile, ctx);
